import { declare } from '@babel/helper-plugin-utils';
import { types, template } from '@babel/core';

var index = declare((api, {
  loose = false
}) => {
  api.assertVersion(7);
  const noDocumentAll = api.assumption("noDocumentAll") ?? loose;
  return {
    name: "transform-nullish-coalescing-operator",
    inherits: undefined,
    visitor: {
      LogicalExpression(path) {
        const {
          node,
          scope
        } = path;
        if (node.operator !== "??") {
          return;
        }
        let ref;
        let assignment;
        if (scope.isStatic(node.left)) {
          ref = node.left;
          assignment = types.cloneNode(node.left);
        } else if (scope.path.isPattern()) {
          path.replaceWith(template.statement.ast`(() => ${path.node})()`);
          return;
        } else {
          ref = scope.generateUidIdentifierBasedOnNode(node.left);
          scope.push({
            id: types.cloneNode(ref)
          });
          assignment = types.assignmentExpression("=", ref, node.left);
        }
        path.replaceWith(types.conditionalExpression(noDocumentAll ? types.binaryExpression("!=", assignment, types.nullLiteral()) : types.logicalExpression("&&", types.binaryExpression("!==", assignment, types.nullLiteral()), types.binaryExpression("!==", types.cloneNode(ref), scope.buildUndefinedNode())), types.cloneNode(ref), node.right));
      }
    }
  };
});

export { index as default };
//# sourceMappingURL=index.js.map
